/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "XMLHttpRequest.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomEvent.h"
#include "gdom/GdomXMLHttpRequest.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomEvent.h"
#include "gdom/PrivateGdomXMLHttpRequest.h"
#include "gdom/PrivateIntGdomEvent.h"
#include "gdom/PrivateIntGdomXMLHttpRequest.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::XMLHttpRequest* obj)
{
    return WebCore::cacheDOMObject<WebCore::XMLHttpRequest, GDOMXMLHttpRequest>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMXMLHttpRequest::GDOMXMLHttpRequest(PassRefPtr<WebCore::XMLHttpRequest> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMXMLHttpRequest::~GDOMXMLHttpRequest()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

void
gdom_xml_http_request_open (GdomXMLHttpRequest *thiz, gchar * method, gchar * url, gboolean async, gchar * user, gchar * password)
{
    g_return_if_fail (thiz);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (method);
    g_return_if_fail (url);
    g_return_if_fail (user);
    g_return_if_fail (password);
    WebCore::String _g_method = WebCore::String::fromUTF8(method);
    WebCore::KURL _g_url = WebCore::KURL(url);
    WebCore::String _g_user = WebCore::String::fromUTF8(user);
    WebCore::String _g_password = WebCore::String::fromUTF8(password);
    WebCore::ExceptionCode ec = 0;
    item->open(_g_method, _g_url, async, _g_user, _g_password, ec);

}

void
gdom_xml_http_request_set_request_header (GdomXMLHttpRequest *thiz, gchar * header, gchar * value)
{
    g_return_if_fail (thiz);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (header);
    g_return_if_fail (value);
    WebCore::String _g_header = WebCore::String::fromUTF8(header);
    WebCore::String _g_value = WebCore::String::fromUTF8(value);
    WebCore::ExceptionCode ec = 0;
    item->setRequestHeader(_g_header, _g_value, ec);

}

void
gdom_xml_http_request_send (GdomXMLHttpRequest *thiz, gchar * data)
{
    g_return_if_fail (thiz);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (data);
    WebCore::String _g_data = WebCore::String::fromUTF8(data);
    WebCore::ExceptionCode ec = 0;
    item->send(_g_data, ec);

}

void
gdom_xml_http_request_abort (GdomXMLHttpRequest *thiz)
{
    g_return_if_fail (thiz);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_if_fail (item);
    item->abort();

}

gchar *
gdom_xml_http_request_get_all_response_headers (GdomXMLHttpRequest *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->getAllResponseHeaders(ec));
    return res;

}

gchar *
gdom_xml_http_request_get_response_header (GdomXMLHttpRequest *thiz, gchar * header)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (header, 0);
    WebCore::String _g_header = WebCore::String::fromUTF8(header);
    WebCore::ExceptionCode ec = 0;
    gchar * res = _G_GET_UTF8(item->getResponseHeader(_g_header, ec));
    return res;

}

void
gdom_xml_http_request_override_mime_type (GdomXMLHttpRequest *thiz, gchar * override)
{
    g_return_if_fail (thiz);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (override);
    WebCore::String _g_override = WebCore::String::fromUTF8(override);
    item->overrideMimeType(_g_override);

}


/* XXX TODO: custom function gdom_xml_http_request_add_event_listener */


/* XXX TODO: custom function gdom_xml_http_request_remove_event_listener */

gboolean
gdom_xml_http_request_dispatch_event (GdomXMLHttpRequest *thiz, GdomEvent * evt)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::XMLHttpRequest * item = (gdom_xml_http_request_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (evt, 0);
    WebCore::Event * _g_evt = (gdom_event_get_private(evt));
    g_return_val_if_fail (_g_evt, 0);
    WebCore::ExceptionCode ec = 0;
    gboolean res = item->dispatchEvent(_g_evt, ec);
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "XMLHttpRequest.h"
#include <gdom/GdomXMLHttpRequest.h>
#include <gdom/PrivateGdomXMLHttpRequest.h>
#include <gdom/PrivateIntGdomXMLHttpRequest.h>
#include <gdom/GdomXMLHttpRequestprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomXMLHttpRequest, gdom_xml_http_request, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomXMLHttpRequestPrivate {
    class WebCore::XMLHttpRequest *item;
};

#define GDOM_XML_HTTP_REQUEST_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_XML_HTTP_REQUEST, GdomXMLHttpRequestPrivate))
*/
G_END_DECLS
WebCore::XMLHttpRequest *
gdom_xml_http_request_get_private(GdomXMLHttpRequest *request)
{
    GDOMXMLHttpRequest *res;
    res = static_cast<GDOMXMLHttpRequest*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::XMLHttpRequest*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_ONABORT,
    PROP_ONERROR,
    PROP_ONLOAD,
    PROP_ONLOADSTART,
    PROP_ONPROGRESS,
    PROP_ONREADYSTATECHANGE,
    PROP_READY_STATE,
    PROP_UPLOAD,
    PROP_RESPONSE_TEXT,
    PROP_RESPONSE_XML,
    PROP_STATUS,
    PROP_STATUS_TEXT,
};


static void gdom_xml_http_request_finalize(GObject* object)
{
    /*GdomXMLHttpRequest* request = GDOM_XML_HTTP_REQUEST(object);*/
    /*GdomXMLHttpRequestPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_xml_http_request_parent_class)->finalize(object);
}

static void gdom_xml_http_request__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomXMLHttpRequest *self = GDOM_XML_HTTP_REQUEST(object);
    WebCore::XMLHttpRequest * gself = (gdom_xml_http_request_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_xml_http_request__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomXMLHttpRequest *self = GDOM_XML_HTTP_REQUEST(object);
    WebCore::XMLHttpRequest * gself = (gdom_xml_http_request_get_private(self));
    switch (prop_id) {
    case PROP_READY_STATE:
    {
         g_value_set_uint (value, gself->readyState());
         break;
    }
    case PROP_UPLOAD:
    {
         PassRefPtr<WebCore::XMLHttpRequestUpload> ptr = gself->upload();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_RESPONSE_TEXT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->responseText()));
         break;
    }
    case PROP_RESPONSE_XML:
    {
         PassRefPtr<WebCore::Document> ptr = gself->responseXML();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_STATUS:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_uint (value, gself->status(ec));
         break;
    }
    case PROP_STATUS_TEXT:
    {
         WebCore::ExceptionCode ec = 0;
         g_value_set_string (value, _G_GET_UTF8(gself->statusText(ec)));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_xml_http_request_class_init(GdomXMLHttpRequestClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_xml_http_request_finalize;
    g_klass->set_property = gdom_xml_http_request__set_property_fn;
    g_klass->get_property = gdom_xml_http_request__get_property_fn;

    g_object_class_install_property (g_klass, PROP_READY_STATE,
            g_param_spec_uint ("ready_state", /* name */
                "xml_http_request_ready_state", /* short description */
                "read-only  gushort XMLHttpRequest.ready_state", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_UPLOAD,
            g_param_spec_object ("upload", /* name */
                "xml_http_request_upload", /* short description */
                "read-only  GdomXMLHttpRequestUpload * XMLHttpRequest.upload", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_XML_HTTP_REQUEST_UPLOAD, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_RESPONSE_TEXT,
            g_param_spec_string ("response_text", /* name */
                "xml_http_request_response_text", /* short description */
                "read-only  gchar * XMLHttpRequest.response_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_RESPONSE_XML,
            g_param_spec_object ("response_xml", /* name */
                "xml_http_request_response_xml", /* short description */
                "read-only  GdomDocument * XMLHttpRequest.response_xml", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOCUMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_STATUS,
            g_param_spec_uint ("status", /* name */
                "xml_http_request_status", /* short description */
                "read-only  gushort XMLHttpRequest.status", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXUINT16, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_STATUS_TEXT,
            g_param_spec_string ("status_text", /* name */
                "xml_http_request_status_text", /* short description */
                "read-only  gchar * XMLHttpRequest.status_text", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));



    event_signals[BROWSER_EVENT] = g_signal_new("browser-event",
        G_TYPE_FROM_CLASS(requestClass),
        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION |
                                           G_SIGNAL_DETAILED  ),
        0,
        NULL,
        NULL,
        webkit_marshal_VOID__OBJECT_BOOLEAN,
        G_TYPE_NONE, 2,
        GDOM_TYPE_EVENT, G_TYPE_BOOLEAN);


    /*g_type_class_remove_private(requestClass, sizeof(GdomXMLHttpRequestPrivate));*/
}

static void gdom_xml_http_request_init(GdomXMLHttpRequest* request)
{
    /*GdomXMLHttpRequestPrivate* priv = GDOM_XML_HTTP_REQUEST_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

